@@ -16,6 +16,13 @@
  * You should have received a copy of the GNU General Public License
  * along with QCAD.
  */
+
+#include <QtGlobal>
+
+#if QT_VERSION >= 0x060000
+#include <QRandomGenerator>
+#endif
+
 #include "RBox.h"
 #include "RDebug.h"
 #include "RDocument.h"
@@ -207,52 +214,53 @@ void RDocument::init(bool beforeLoad) {
         docVars->setKnownVariable(RS::PDSIZE, RSettings::getDoubleValue("PointDisplaySettings/Size", 0.0));
 
         // dimension properties:
-        docVars->setKnownVariable(RS::DIMTXT, RSettings::getDoubleValue("DimensionSettings/DIMTXT", 2.5));
-        docVars->setKnownVariable(RS::DIMEXE, RSettings::getDoubleValue("DimensionSettings/DIMEXE", 1.25));
-        docVars->setKnownVariable(RS::DIMEXO, RSettings::getDoubleValue("DimensionSettings/DIMEXO", 0.625));
-        docVars->setKnownVariable(RS::DIMGAP, RSettings::getDoubleValue("DimensionSettings/DIMGAP", 0.625));
-        docVars->setKnownVariable(RS::DIMASZ, RSettings::getDoubleValue("DimensionSettings/DIMASZ", 2.5));
-        docVars->setKnownVariable(RS::DIMSCALE, RSettings::getDoubleValue("DimensionSettings/DIMSCALE", 1.0));
-        docVars->setKnownVariable(RS::DIMDLI, RSettings::getDoubleValue("DimensionSettings/DIMDLI", 5.0));
-        docVars->setKnownVariable(RS::DIMCLRT, RSettings::getColorValue("DimensionSettings/DimensionTextColor", RColor(RColor::ByBlock)));
+        // TODO: change defaults to defaults from RDimStyle::getDoubleDefault...
+//        docVars->setKnownVariable(RS::DIMTXT, RSettings::getDoubleValue("DimensionSettings/DIMTXT", 2.5));
+//        docVars->setKnownVariable(RS::DIMEXE, RSettings::getDoubleValue("DimensionSettings/DIMEXE", 1.25));
+//        docVars->setKnownVariable(RS::DIMEXO, RSettings::getDoubleValue("DimensionSettings/DIMEXO", 0.625));
+//        docVars->setKnownVariable(RS::DIMGAP, RSettings::getDoubleValue("DimensionSettings/DIMGAP", 0.625));
+//        docVars->setKnownVariable(RS::DIMASZ, RSettings::getDoubleValue("DimensionSettings/DIMASZ", 2.5));
+//        docVars->setKnownVariable(RS::DIMSCALE, RSettings::getDoubleValue("DimensionSettings/DIMSCALE", 1.0));
+//        docVars->setKnownVariable(RS::DIMDLI, RSettings::getDoubleValue("DimensionSettings/DIMDLI", 5.0));
+//        docVars->setKnownVariable(RS::DIMCLRT, RSettings::getColorValue("DimensionSettings/DimensionTextColor", RColor(RColor::ByBlock)));
 
         // arrow head:
-        if (RSettings::getStringValue("DimensionSettings/ArrowStyle", "Arrow")=="Arrow") {
-            // tick size is 0 for arrows:
-            docVars->setKnownVariable(RS::DIMTSZ, 0.0);
-        }
+//        if (RSettings::getStringValue("DimensionSettings/ArrowStyle", "Arrow")=="Arrow") {
+//            // tick size is 0 for arrows:
+//            docVars->setKnownVariable(RS::DIMTSZ, 0.0);
+//        }
 
-        // arch tick head:
-        else {
-            docVars->setKnownVariable(RS::DIMTSZ, RSettings::getDoubleValue("DimensionSettings/DIMASZ", 2.5));
-        }
+//        // arch tick head:
+//        else {
+//            docVars->setKnownVariable(RS::DIMTSZ, RSettings::getDoubleValue("DimensionSettings/DIMASZ", 2.5));
+//        }
 
-        docVars->setKnownVariable(RS::DIMLUNIT, RSettings::getIntValue("DimensionSettings/LinearFormat", RS::Decimal));
-        docVars->setKnownVariable(RS::DIMDEC, RSettings::getIntValue("DimensionSettings/LinearPrecision", 4));
-        docVars->setKnownVariable(RS::DIMDSEP, RSettings::getIntValue("DimensionSettings/DecimalPoint", '.'));
+//        docVars->setKnownVariable(RS::DIMLUNIT, RSettings::getIntValue("DimensionSettings/LinearFormat", RS::Decimal));
+//        docVars->setKnownVariable(RS::DIMDEC, RSettings::getIntValue("DimensionSettings/LinearPrecision", 4));
+//        docVars->setKnownVariable(RS::DIMDSEP, RSettings::getIntValue("DimensionSettings/DecimalPoint", '.'));
 
         // show trailing zeroes:
-        if (RSettings::getBoolValue("DimensionSettings/LinearShowTrailingZeros", false)) {
-            docVars->setKnownVariable(RS::DIMZIN, 0);
-        }
+//        if (RSettings::getBoolValue("DimensionSettings/LinearShowTrailingZeros", false)) {
+//            docVars->setKnownVariable(RS::DIMZIN, 0);
+//        }
 
-        // suppress trailing zeroes:
-        else {
-            docVars->setKnownVariable(RS::DIMZIN, 8);
-        }
+//        // suppress trailing zeroes:
+//        else {
+//            docVars->setKnownVariable(RS::DIMZIN, 8);
+//        }
 
-        docVars->setKnownVariable(RS::DIMAUNIT, RSettings::getIntValue("DimensionSettings/AngularFormat", RS::DegreesDecimal));
-        docVars->setKnownVariable(RS::DIMADEC, RSettings::getIntValue("DimensionSettings/AngularPrecision", 0));
+//        docVars->setKnownVariable(RS::DIMAUNIT, RSettings::getIntValue("DimensionSettings/AngularFormat", RS::DegreesDecimal));
+//        docVars->setKnownVariable(RS::DIMADEC, RSettings::getIntValue("DimensionSettings/AngularPrecision", 0));
 
         // show trailing zeroes:
-        if (RSettings::getBoolValue("DimensionSettings/AngularShowTrailingZeros", false)) {
-            docVars->setKnownVariable(RS::DIMAZIN, 0);
-        }
+//        if (RSettings::getBoolValue("DimensionSettings/AngularShowTrailingZeros", false)) {
+//            docVars->setKnownVariable(RS::DIMAZIN, 0);
+//        }
 
-        // suppress trailing zeroes:
-        else {
-            docVars->setKnownVariable(RS::DIMAZIN, 2);
-        }
+//        // suppress trailing zeroes:
+//        else {
+//            docVars->setKnownVariable(RS::DIMAZIN, 2);
+//        }
 
         // max number of active viewports:
         docVars->setKnownVariable(RS::MAXACTVP, 64);
@@ -273,7 +281,11 @@ void RDocument::init(bool beforeLoad) {
         setVariable("ColorSettings/ColorMode", RSettings::getStringValue("ColorSettings/ColorMode", "FullColor"));
         RColor col = RSettings::getColorValue("ColorSettings/BackgroundColor", RColor(QString("white")));
         QVariant v;
+#if QT_VERSION >= 0x060000
+        v.setValue(col);
+#else
         v.setValue<RColor>(col);
+#endif
         setVariable("ColorSettings/BackgroundColor", v);
 
         // printer page settings:
@@ -343,6 +355,9 @@ void RDocument::init(bool beforeLoad) {
 
         docVars->setDimensionFont(RSettings::getStringValue("DimensionSettings/DimensionFont", "Standard"));
 
+        // make sure by default, text and arrows are inside / outside:
+        docVars->setKnownVariable(RS::DIMATFIT, 0);
+
         // notify new document listeners
         if (RMainWindow::hasMainWindow()) {
             RMainWindow::getMainWindow()->notifyNewDocumentListeners(this, &transaction, beforeLoad);
@@ -351,6 +366,9 @@ void RDocument::init(bool beforeLoad) {
 
     transaction.addObject(docVars);
 
+    QSharedPointer<RDimStyle> dimStyle = QSharedPointer<RDimStyle>(new RDimStyle(this));
+    transaction.addObject(dimStyle);
+
     transaction.end();
     resetTransactionStack();
     storage.setModified(false);
@@ -548,11 +566,17 @@ QString RDocument::formatAngle(double value) {
  * This is determined by the variable "$DIMLUNIT".
  */
 RS::LinearFormat RDocument::getLinearFormat() const {
-    return (RS::LinearFormat)getKnownVariable(RS::DIMLUNIT, 2).toInt();
+    //return (RS::LinearFormat)getKnownVariable(RS::DIMLUNIT, 2).toInt();
+
+    QSharedPointer<RDimStyle> dimStyle = queryDimStyleDirect();
+    return (RS::LinearFormat)dimStyle->getInt(RS::DIMLUNIT);
 }
 
 void RDocument::setLinearFormat(RS::LinearFormat f) {
-    setKnownVariable(RS::DIMLUNIT, f);
+    //setKnownVariable(RS::DIMLUNIT, f);
+
+    QSharedPointer<RDimStyle> dimStyle = queryDimStyleDirect();
+    dimStyle->setInt(RS::DIMLUNIT, f);
 }
 
 
@@ -561,27 +585,42 @@ void RDocument::setLinearFormat(RS::LinearFormat f) {
  * This is determined by the variable "$DIMDEC".
  */
 int RDocument::getLinearPrecision() {
-    return getKnownVariable(RS::DIMDEC, 4).toInt();
+    //return getKnownVariable(RS::DIMDEC, 4).toInt();
+
+    QSharedPointer<RDimStyle> dimStyle = queryDimStyleDirect();
+    return (RS::LinearFormat)dimStyle->getInt(RS::DIMDEC);
 }
 
 
 
 bool RDocument::showLeadingZeroes() {
-    return !(getKnownVariable(RS::DIMZIN, 8).toInt() & 4);
+    //return !(getKnownVariable(RS::DIMZIN, 8).toInt() & 4);
+
+    QSharedPointer<RDimStyle> dimStyle = queryDimStyleDirect();
+    return !(dimStyle->getInt(RS::DIMZIN) & 4);
 }
 
 
 bool RDocument::showTrailingZeroes() {
-    return !(getKnownVariable(RS::DIMZIN, 8).toInt() & 8);
+    //return !(getKnownVariable(RS::DIMZIN, 8).toInt() & 8);
+
+    QSharedPointer<RDimStyle> dimStyle = queryDimStyleDirect();
+    return !(dimStyle->getInt(RS::DIMZIN) & 8);
 }
 
 bool RDocument::showLeadingZeroesAngle() {
-    return !(getKnownVariable(RS::DIMAZIN, 3).toInt() & 1);
+    //return !(getKnownVariable(RS::DIMAZIN, 3).toInt() & 1);
+
+    QSharedPointer<RDimStyle> dimStyle = queryDimStyleDirect();
+    return !(dimStyle->getInt(RS::DIMAZIN) & 1);
 }
 
 
 bool RDocument::showTrailingZeroesAngle() {
-    return !(getKnownVariable(RS::DIMAZIN, 3).toInt() & 2);
+    //return !(getKnownVariable(RS::DIMAZIN, 3).toInt() & 2);
+
+    QSharedPointer<RDimStyle> dimStyle = queryDimStyleDirect();
+    return !(dimStyle->getInt(RS::DIMAZIN) & 2);
 }
 
 
@@ -590,7 +629,10 @@ bool RDocument::showTrailingZeroesAngle() {
  * This is determined by the variable "$DIMAUNIT".
  */
 RS::AngleFormat RDocument::getAngleFormat() {
-    return (RS::AngleFormat)getKnownVariable(RS::DIMAUNIT, 0).toInt();
+    //return (RS::AngleFormat)getKnownVariable(RS::DIMAUNIT, 0).toInt();
+
+    QSharedPointer<RDimStyle> dimStyle = queryDimStyleDirect();
+    return (RS::AngleFormat)dimStyle->getInt(RS::DIMAUNIT);
 }
 
 /**
@@ -598,7 +640,10 @@ RS::AngleFormat RDocument::getAngleFormat() {
  * This is determined by the variable "$DIMADEC".
  */
 int RDocument::getAnglePrecision() {
-    return getKnownVariable(RS::DIMADEC, 2).toInt();
+    //return getKnownVariable(RS::DIMADEC, 2).toInt();
+
+    QSharedPointer<RDimStyle> dimStyle = queryDimStyleDirect();
+    return (RS::AngleFormat)dimStyle->getInt(RS::DIMADEC);
 }
 
 /**
@@ -607,7 +652,14 @@ int RDocument::getAnglePrecision() {
  * 0 means '.'.
  */
 char RDocument::getDecimalSeparator() {
-    int ret = (char)getKnownVariable(RS::DIMDSEP, (int)'.').toInt();
+//    int ret = (char)getKnownVariable(RS::DIMDSEP, (int)'.').toInt();
+//    if (ret==0) {
+//        ret = '.';
+//    }
+//    return ret;
+
+    QSharedPointer<RDimStyle> dimStyle = queryDimStyleDirect();
+    int ret = dimStyle->getInt(RS::DIMDSEP);
     if (ret==0) {
         ret = '.';
     }
@@ -878,8 +930,13 @@ void RDocument::unsetCurrentViewport() {
 
 QString RDocument::getTempBlockName() const {
     do {
+#if QT_VERSION >= 0x060000
+        int p1 = QRandomGenerator::global()->generate() % 100000;
+        int p2 = QRandomGenerator::global()->generate() % 100000;
+#else
         int p1 = qrand() % 100000;
         int p2 = qrand() % 100000;
+#endif
         QString blockName = QString("A$C%1%2").arg(p1, 5, 10, QChar('0')).arg(p2, 5, 10, QChar('0'));
         if (!hasBlock(blockName)) {
             return blockName;
@@ -894,6 +951,10 @@ QString RDocument::getBlockName(RBlock::Id blockId) const {
     return storage.getBlockName(blockId);
 }
 
+QString RDocument::getBlockNameFromHandle(RBlock::Handle blockHandle) const {
+    return storage.getBlockNameFromHandle(blockHandle);
+}
+
 QString RDocument::getBlockNameFromLayout(const QString& layoutName) const {
     return storage.getBlockNameFromLayout(layoutName);
 }
@@ -1345,6 +1406,13 @@ QSet<REntity::Id> RDocument::queryLayerEntities(RLayer::Id layerId, bool allBloc
     return storage.queryLayerEntities(layerId, allBlocks);
 }
 
+/**
+ * \copydoc RStorage::querySelectedLayerEntities
+ */
+QSet<REntity::Id> RDocument::querySelectedLayerEntities(RLayer::Id layerId, bool allBlocks) const {
+    return storage.querySelectedLayerEntities(layerId, allBlocks);
+}
+
 /**
  * \copydoc RStorage::hasBlockEntities
  */
@@ -1447,6 +1515,39 @@ REntity::Id RDocument::queryClosestXY(
     return queryClosestXY(candidates, wcsPosition, range, draft, strictRange);
 }
 
+QPair<REntity::Id, QSet<int> > RDocument::queryClosestXYWithIndices(
+        const RVector& wcsPosition,
+        double range,
+        bool draft,
+        double strictRange,
+        bool includeLockedLayers,
+        bool selectedOnly) {
+
+    RVector rangeV(
+                range,
+                range
+                );
+
+    // find entities that are within given maximum distance
+    //   (approximation based on bounding boxes):
+    QMap<REntity::Id, QSet<int> > candidates =
+            queryIntersectedEntitiesXYWithIndex(
+                RBox(
+                    wcsPosition - rangeV,
+                    wcsPosition + rangeV
+                    ),
+                true, includeLockedLayers,
+                RBlock::INVALID_ID, RDEFAULT_QLIST_RS_ENTITYTYPE,
+                selectedOnly
+                );
+
+    if (candidates.isEmpty()) {
+        return QPair<REntity::Id, QSet<int> >(REntity::INVALID_ID, QSet<int>());
+    }
+
+    return queryClosestXYWithIndices(candidates, wcsPosition, range, draft, strictRange);
+}
+
 /**
  * Queries the entity that is closest to the given position \c wcsPosition.
  * Only entities in the given set of \c candidates are considered.
@@ -1492,6 +1593,44 @@ REntity::Id RDocument::queryClosestXY(
     return ret;
 }
 
+QPair<REntity::Id, QSet<int> > RDocument::queryClosestXYWithIndices(
+        QMap<REntity::Id, QSet<int> >& candidates,
+        const RVector& wcsPosition,
+        double range,
+        bool draft,
+        double strictRange) {
+
+    double minDist = RMAXDOUBLE;
+    QPair<REntity::Id, QSet<int> > ret = QPair<REntity::Id, QSet<int> >(REntity::INVALID_ID, QSet<int>());
+
+    QMap<REntity::Id, QSet<int> >::iterator it;
+    for (it=candidates.begin(); it!=candidates.end(); it++) {
+        if (RMouseEvent::hasMouseMoved()) {
+            return QPair<REntity::Id, QSet<int> >(REntity::INVALID_ID, QSet<int>());
+        }
+        QSharedPointer<REntity> e = queryEntityDirect(it.key());
+        if (e.isNull()) {
+            continue;
+        }
+        double dist = e->getDistanceTo(wcsPosition, true, range, draft, strictRange);
+
+        if (e->isPointType() && dist<strictRange*1.1) {
+            // make point type entities easier to select:
+            dist/=100.0;
+        }
+
+        if (!RMath::isNaN(dist) && dist < minDist && dist < range+RS::PointTolerance) {
+            minDist = dist;
+            //QSet<int> val = it.value();
+            //ret = QPair<REntity::Id, QSet<int> >(it.key(), val);
+            ret.first = it.key();
+            ret.second = it.value();
+        }
+    }
+
+    return ret;
+}
+
 QSet<REntity::Id> RDocument::queryInfiniteEntities() const {
     return storage.queryInfiniteEntities();
 }
@@ -1506,7 +1645,7 @@ QSet<REntity::Id> RDocument::queryInfiniteEntities() const {
  */
 QSet<REntity::Id> RDocument::queryContainedEntities(const RBox& box) const {
     RSpatialIndex* si = getSpatialIndexForCurrentBlock();
-    QSet<REntity::Id> ret = si->queryContained(box).keys().toSet();
+    QSet<REntity::Id> ret = RS::toSet<REntity::Id>(si->queryContained(box).keys());
 
     // always exclude construction lines (XLine):
     ret.subtract(queryInfiniteEntities());
@@ -1547,7 +1686,7 @@ QSet<REntity::Id> RDocument::queryIntersectedShapesXYFast(const RBox& box, bool
 
     RSpatialIndex* si = getSpatialIndexForBlock(getCurrentBlockId());
     //RDebug::startTimer(120);
-    QSet<REntity::Id> candidates = si->queryIntersected(box).keys().toSet();
+    QSet<REntity::Id> candidates = RS::toSet<REntity::Id>(si->queryIntersected(box).keys());
     //RDebug::stopTimer(120, "si->queryIntersected");
     candidates.unite(infinites);
 
@@ -1578,6 +1717,13 @@ QSet<REntity::Id> RDocument::queryIntersectedEntitiesXY(
         const RBox& box, bool checkBoundingBoxOnly, bool includeLockedLayers, RBlock::Id blockId,
         const QList<RS::EntityType>& filter, bool selectedOnly, RLayer::Id layerId) const {
 
+    return RS::toSet<REntity::Id>(queryIntersectedEntitiesXYWithIndex(box, checkBoundingBoxOnly, includeLockedLayers, blockId, filter, selectedOnly, layerId).keys());
+}
+
+QMap<REntity::Id, QSet<int> > RDocument::queryIntersectedEntitiesXYWithIndex(
+        const RBox& box, bool checkBoundingBoxOnly, bool includeLockedLayers, RBlock::Id blockId,
+        const QList<RS::EntityType>& filter, bool selectedOnly, RLayer::Id layerId) const {
+
     bool onlyVisible = false;
 
     RBox boxExpanded = box;
@@ -1604,10 +1750,16 @@ QSet<REntity::Id> RDocument::queryIntersectedEntitiesXY(
             ids = queryAllEntities(false, false);
         }
 
-        return ids;
+        QMap<REntity::Id, QSet<int> > ret;
+        QSet<REntity::Id>::iterator it;
+        for (it=ids.begin(); it!=ids.end(); it++) {
+            QSet<int> val;
+            ret.insert(*it, val);
+        }
+        return ret;
     }
 
-    return queryIntersectedShapesXY(box, checkBoundingBoxOnly, includeLockedLayers, blockId, filter, selectedOnly, layerId).keys().toSet();
+    return queryIntersectedShapesXY(box, checkBoundingBoxOnly, includeLockedLayers, blockId, filter, selectedOnly, layerId);
 }
 
 QMap<REntity::Id, QSet<int> > RDocument::queryIntersectedShapesXY(
@@ -1616,7 +1768,7 @@ QMap<REntity::Id, QSet<int> > RDocument::queryIntersectedShapesXY(
 
     bool onlyVisible = false;
 
-    QSet<RS::EntityType> filterSet = filter.toSet();
+    QSet<RS::EntityType> filterSet = RS::toSet<RS::EntityType>(filter);
     RBox boxExpanded = box;
     boxExpanded.c1.z = RMINDOUBLE;
     boxExpanded.c2.z = RMAXDOUBLE;
@@ -1663,7 +1815,13 @@ QMap<REntity::Id, QSet<int> > RDocument::queryIntersectedShapesXY(
     else {
         RSpatialIndex* si = getSpatialIndexForBlock(blockId);
         candidates = si->queryIntersected(boxExpanded);
-        candidates.unite(infinites);
+
+        //candidates.unite(infinites);
+
+        QMap<REntity::Id, QSet<int> >::iterator it;
+        for (it=infinites.begin(); it!=infinites.end(); it++) {
+            candidates.insert(it.key(), it.value());
+        }
     }
 
     RBox boxFlattened = box;
@@ -2012,6 +2170,12 @@ QSet<RObject::Id> RDocument::queryPropertyEditorObjects() {
 //                    }
                 }
             }
+
+            // dim style properties, not shown for now:
+//            QSharedPointer<RDimStyle> dimStyle = queryDimStyleDirect();
+//            if (!dimStyle.isNull()) {
+//                objectIds.insert(dimStyle->getId());
+//            }
         }
     }
 
@@ -2026,6 +2190,14 @@ QSharedPointer<RDocumentVariables> RDocument::queryDocumentVariablesDirect() con
     return storage.queryDocumentVariablesDirect();
 }
 
+QSharedPointer<RDimStyle> RDocument::queryDimStyle() const {
+    return storage.queryDimStyle();
+}
+
+QSharedPointer<RDimStyle> RDocument::queryDimStyleDirect() const {
+    return storage.queryDimStyleDirect();
+}
+
 /**
  * Queries the object with the given ID.
  *
@@ -2048,6 +2220,10 @@ QSharedPointer<RObject> RDocument::queryObjectDirect(RObject::Id objectId) const
     return storage.queryObjectDirect(objectId);
 }
 
+RObject* RDocument::queryObjectCC(RObject::Id objectId) const {
+    return storage.queryObjectCC(objectId);
+}
+
 QSharedPointer<RObject> RDocument::queryObjectByHandle(RObject::Handle objectHandle) const {
     return storage.queryObjectByHandle(objectHandle);
 }
@@ -2305,6 +2481,14 @@ bool RDocument::isParentLayerLocked(const RLayer& layer) const {
     return storage.isParentLayerLocked(layer);
 }
 
+bool RDocument::isEntity(RObject::Id entityId) const {
+    QSharedPointer<REntity> entity = queryEntityDirect(entityId);
+    if (entity.isNull()) {
+        return false;
+    }
+    return true;
+}
+
 bool RDocument::isEntityEditable(REntity::Id entityId) const {
     QSharedPointer<REntity> entity = queryEntityDirect(entityId);
     if (entity.isNull()) {
@@ -2625,7 +2809,7 @@ void RDocument::rebuildSpatialIndex() {
 
 
     if (!disableSpatialIndicesByBlock) {
-        QList<RBlock::Id> blockIds = queryAllBlocks().toList();
+        QList<RBlock::Id> blockIds = RS::toList<RBlock::Id>(queryAllBlocks());
         for (int i=0; i<blockIds.length(); i++) {
             RBlock::Id blockId = blockIds[i];
             RSpatialIndex* si = getSpatialIndexForBlock(blockId);
@@ -2899,7 +3083,7 @@ QString RDocument::substituteAutoVariables(const QString& expression) {
         if (RMath::isNaN(value)) {
             continue;
         }
-        exp = exp.replace(QRegExp(QString("\\b%1\\b").arg(key)), QString("%1").arg(value, 0, 'f', 12));
+        exp = exp.replace(QRegularExpression(QString("\\b%1\\b").arg(key)), QString("%1").arg(value, 0, 'f', 12));
     }
 
     return exp;
@@ -2935,6 +3119,14 @@ void RDocument::setIgnoreWorkingSet(bool on) {
     }
 }
 
+void RDocument::setNotifyListeners(bool on) {
+    getStorage().setNotifyListeners(on);
+}
+
+bool RDocument::getNotifyListeners() const {
+    return getStorage().getNotifyListeners();
+}
+
 //RBlockReferenceEntity::Id RDocument::getWorkingSetBlockReferenceId() const {
 //    return storage.getWorkingSetBlockReferenceId();
 //}
@@ -2947,7 +3139,7 @@ void RDocument::setIgnoreWorkingSet(bool on) {
  * Stream operator for QDebug
  */
 QDebug operator<<(QDebug dbg, RDocument& d) {
-    dbg.nospace() << "RDocument(" << QString("0x%1").arg((long int)&d, 0, 16) << ", ";
+    dbg.nospace() << "RDocument(" << QString("0x%1").arg((unsigned long long int)&d, 0, 16) << ", ";
     dbg.nospace() << d.getStorage();
     dbg.nospace() << d.getSpatialIndex();
 //    QSet<RBlock::Id> blockIds = d.queryAllBlocks();
