@@ -23,7 +23,7 @@
 #include <QPainter>
 #include <QSvgRenderer>
 #include <QToolBar>
-#include <QVectorIterator>
+//#include <QVectorIterator>
 
 #include "RDebug.h"
 #include "RGuiAction.h"
@@ -82,7 +82,23 @@ RGuiAction::~RGuiAction() {
     maps << &actionsByShortcut;
     maps << &actionsByPrimaryCommand;
     maps << &actionsByScriptFile;
+#if QT_VERSION >= 0x060000
+    QMultiMap<QString, RGuiAction*>::iterator it;
+    QStringList keys;
+    for (it=actionsByGroup.begin(); it!=actionsByGroup.end(); it++) {
+        if (it.value() == this) {
+            keys << it.key();
+        }
+    }
+    keys.removeDuplicates();
+    QStringListIterator ki(keys);
+    while (ki.hasNext()) {
+        QString s = ki.next();
+        actionsByGroup.remove(s);
+    }
+#else
     maps << &actionsByGroup;
+#endif
 
     QListIterator<QMap<QString, RGuiAction*>*> vi(maps);
     while (vi.hasNext()) {
@@ -135,7 +151,7 @@ void RGuiAction::initTexts() {
             textAndKeycode += " (" + shortcutText + ")";
         }
 #else
-        // tab does not work for Mac OS X:
+        // tab does not work for macOS:
         if (textAndKeycode.indexOf(QLatin1Char('\t'))!=-1) {
             textAndKeycode = textAndKeycode.left(textAndKeycode.indexOf('\t'));
         }
@@ -146,7 +162,7 @@ void RGuiAction::initTexts() {
 
     QAction::setText(textAndKeycode);
 
-    QString tip = toolTip;
+    QString tip = oriToolTip;
     if (tip.isNull()) {
         tip = textOnly;
     }
@@ -422,7 +438,7 @@ QList<QKeySequence> RGuiAction::getShortcuts() const {
 }
 
 void RGuiAction::setToolTip(const QString& tip) {
-    toolTip = tip;
+    oriToolTip = tip;
     initTexts();
 }
 
@@ -560,12 +576,13 @@ void RGuiAction::setWidgetNamesStatic(QAction* a, const QStringList& widgetNames
 }
 
 QStringList RGuiAction::getWidgetNamesStatic(const QAction* a) {
+    QStringList ret;
     if (a->property("WidgetNames").isValid()) {
-        return a->property("WidgetNames").toStringList();
-    }
-    else {
-        return QStringList();
+        ret = a->property("WidgetNames").toStringList();
     }
+    ret.append("!UserToolBar1");
+    ret.append("!UserToolBar2");
+    return ret;
 }
 
 void RGuiAction::setWidgetNames(const QStringList& widgetNames) {
@@ -663,6 +680,12 @@ void RGuiAction::addToWidget(QAction* action, QWidget* w) {
 
     RWidget* rw = dynamic_cast<RWidget*>(w);
 
+    if (action->isSeparator() && rw!=NULL) {
+        // don't add separator to CAD toolbar (handled differently, by RColumnLayout):
+        delete action;
+        return;
+    }
+
     // find existing groups:
     QSet<int> groupSortOrders;
     QList<QAction*> actions = w->actions();
@@ -780,7 +803,8 @@ void RGuiAction::updateTransactionListener(RDocument* document, RTransaction* tr
             if (undoText.isEmpty()) {
                 undoText = "-";
             }
-            t.replace(QRegExp("\\[[^\\]]*\\]"), "[" + undoText + "]");
+            undoText.replace("\t", " ");
+            t.replace(QRegularExpression("\\[[^\\]]*\\]"), "[" + undoText + "]");
             setText(t);
         }
     }
@@ -792,7 +816,8 @@ void RGuiAction::updateTransactionListener(RDocument* document, RTransaction* tr
             if (redoText.isEmpty()) {
                 redoText = "-";
             }
-            t.replace(QRegExp("\\[[^\\]]*\\]"), "[" + redoText + "]");
+            redoText.replace("\t", " ");
+            t.replace(QRegularExpression("\\[[^\\]]*\\]"), "[" + redoText + "]");
             setText(t);
         }
     }
@@ -934,7 +959,7 @@ void RGuiAction::triggerGroupDefault(const QString& group) {
 }
 
 void RGuiAction::triggerGroupDefaults() {
-    QStringList groups = actionsByGroup.keys().toSet().toList();
+    QStringList groups = RS::unique<QString>(actionsByGroup.keys());
     for (int i=0; i<groups.length(); i++) {
         triggerGroupDefault(groups.at(i));
     }
