@@ -30,6 +30,7 @@
 #include "RShape.h"
 #include "RSpline.h"
 #include "RSplineProxy.h"
+#include "RTriangle.h"
 #include "RXLine.h"
 
 double RShape::twopi = M_PI*2;
@@ -221,10 +222,10 @@ QList<RVector> RShape::getIntersectionPoints(const RShape& shape1,
     QList<RVector> empty;
 
     bool gotInfiniteShape = false;
-    if (typeid(shape1) == typeid(RXLine) ||
-        typeid(shape2) == typeid(RXLine) ||
-        typeid(shape1) == typeid(RRay) ||
-        typeid(shape2) == typeid(RRay)) {
+    if (shape1.getShapeType() == RShape::XLine ||
+        shape2.getShapeType() == RShape::XLine ||
+        shape1.getShapeType() == RShape::Ray ||
+        shape2.getShapeType() == RShape::Ray) {
 
         gotInfiniteShape = true;
     }
@@ -1059,19 +1060,22 @@ QList<RVector> RShape::getIntersectionPointsCC(const RCircle& circle1, const RCi
     RVector u = c2 - c1;
     double uMag = u.getMagnitude();
 
+    // concentric
+    if (uMag < RS::PointTolerance) {
+        return res;
+    }
+
+    double tol = (r1+r2)/200000;
+
     // the two circles (almost) touch externally / internally in one point (tangent):
-    if (RMath::fuzzyCompare(uMag, r1+r2, 1.0e-4) ||
-        RMath::fuzzyCompare(uMag, fabs(r1-r2), 1.0e-4)) {
+    if (RMath::fuzzyCompare(uMag, r1+r2, tol) ||
+        RMath::fuzzyCompare(uMag, fabs(r1-r2), tol)) {
 
         u.setMagnitude2D(r1);
         res.append(c1 + u);
         return res;
     }
 
-    // concentric
-    if (uMag < RS::PointTolerance) {
-        return res;
-    }
 
     RVector v = RVector(u.y, -u.x);
 
@@ -1093,7 +1097,7 @@ QList<RVector> RShape::getIntersectionPointsCC(const RCircle& circle1, const RCi
     RVector sol1 = c1 + u*s + v*t1;
     RVector sol2 = c1 + u*s + v*t2;
 
-    if (sol1.equalsFuzzy(sol2, 1.0e-4)) {
+    if (sol1.equalsFuzzy(sol2, tol)) {
         res.append(sol1);
     }
     else {
@@ -1677,11 +1681,11 @@ QList<RVector> RShape::getIntersectionPointsSX(const RSpline& spline1,
 }
 
 QList<RVector> RShape::getIntersectionPointsSS(const RSpline& spline1,
-        const RSpline& spline2, bool limited, bool same) {
+        const RSpline& spline2, bool limited, bool same, double tolerance) {
 
     if (RSpline::hasProxy()) {
         RSplineProxy* proxy = RSpline::getSplineProxy();
-        return proxy->getIntersectionPoints(spline1, spline2, limited, same);
+        return proxy->getIntersectionPoints(spline1, spline2, limited, same, tolerance);
     }
 
     return QList<RVector>();
@@ -1758,10 +1762,104 @@ bool RShape::scale(double scaleFactor, const RVector& center) {
 
 void RShape::print(QDebug dbg) const {
     dbg.nospace() << "RShape("
-            << "address: " << QString("0x%1").arg((long int) this, 0, 16)
+            << "address: " << QString("0x%1").arg((unsigned long long int) this, 0, 16)
             << ")";
 }
 
+QList<QSharedPointer<RShape> > RShape::getOrderedShapes(const QList<QSharedPointer<RShape> >& shapes) {
+    QList<QSharedPointer<RShape> > ret;
+    //RVector cursor = RVector::invalid;
+    QSet<int> traversed;
+
+    // find next first entity of a contour:
+    for (int i=0; i<shapes.size(); ++i) {
+        if (traversed.contains(i)) {
+            continue;
+        }
+
+        QSharedPointer<RShape> shape = shapes.at(i);
+        //qDebug() << "RShape::order: shape: " << *shape;
+
+        // circle:
+        QSharedPointer<RCircle> circle = shape.dynamicCast<RCircle>();
+        if (!circle.isNull()) {
+            ret.append(shape);
+            traversed.insert(i);
+            continue;
+        }
+
+        // full ellipse:
+        QSharedPointer<REllipse> ellipseArc = shape.dynamicCast<REllipse>();
+        if (!ellipseArc.isNull() && ellipseArc->isFullEllipse()) {
+            ret.append(shape);
+            traversed.insert(i);
+            continue;
+        }
+
+        // full spline:
+        QSharedPointer<RSpline> spline = shape.dynamicCast<RSpline>();
+        if (!spline.isNull()) {
+            if (spline->isPeriodic()) {
+                ret.append(shape);
+                traversed.insert(i);
+                continue;
+            }
+        }
+
+        // line, arc, ellipse arc, spline loop segment:
+        if (shape->isDirected()) {
+            QList<QSharedPointer<RShape> > contour;
+            RVector sp = shape->getStartPoint();
+            RVector ep = shape->getEndPoint();
+            bool found = false;
+
+            // first shape of contour:
+            contour.append(shape);
+            traversed.insert(i);
+
+            // extend contour on both sides:
+            do {
+                found = false;
+                for (int k=0; k<shapes.size(); ++k) {
+                    if (traversed.contains(k)) {
+                        continue;
+                    }
+
+                    QSharedPointer<RShape> shape2 = shapes.at(k);
+                    if (ep.equalsFuzzy(shape2->getStartPoint())) {
+                        contour.append(shape2);
+                        traversed.insert(k);
+                        found = true;
+                    }
+                    if (ep.equalsFuzzy(shape2->getEndPoint())) {
+                        RShape* s = shape2->clone();
+                        s->reverse();
+                        contour.append(QSharedPointer<RShape>(s));
+                        traversed.insert(k);
+                        found = true;
+                    }
+                    if (sp.equalsFuzzy(shape2->getEndPoint())) {
+                        contour.prepend(shape2);
+                        traversed.insert(k);
+                        found = true;
+                    }
+                    if (sp.equalsFuzzy(shape2->getStartPoint())) {
+                        RShape* s = shape2->clone();
+                        s->reverse();
+                        contour.append(QSharedPointer<RShape>(s));
+                        traversed.insert(k);
+                        found = true;
+                    }
+                }
+            } while(found);
+
+            ret.append(contour);
+        }
+    }
+
+    return ret;
+}
+
 /**
  * Orders the given list of loops into a list of closed loops in which all
  * shapes are connected end to start point.
@@ -2325,7 +2423,7 @@ QList<QSharedPointer<RShape> > RShape::trim(
 /**
  * Round every corner of the list of given (connected) shapes with the given radius.
  */
-QList<QSharedPointer<RShape> > RShape::roundAllCorners(const QList<QSharedPointer<RShape> >& shapes, double radius) {
+QList<QSharedPointer<RShape> > RShape::roundCorners(const QList<QSharedPointer<RShape> >& shapes, double radius) {
     if (RShape::hasProxy()) {
         return RShape::getShapeProxy()->roundAllCorners(shapes, radius);
     }
@@ -2410,13 +2508,13 @@ QList<QSharedPointer<RShape> > RShape::roundShapes(
     QSharedPointer<RShape> parallel1 = parallels1[0];
     QSharedPointer<RShape> parallel2 = parallels2[0];
 
-    QList<RVector> sol2 = simpleShape1->getIntersectionPoints(*simpleShape2.data(), false);
-
     QList<RVector> ipParallel = parallel1->getIntersectionPoints(*parallel2.data(), false);
     if (ipParallel.isEmpty()) {
         return ret;
     }
 
+    QList<RVector> sol2 = simpleShape1->getIntersectionPoints(*simpleShape2.data(), false);
+
     // there might be two intersections: choose the closest:
     RVector ip = p.getClosest(ipParallel);
     RVector p1 = simpleShape1->getClosestPointOnShape(ip, false);
@@ -2440,7 +2538,8 @@ QList<QSharedPointer<RShape> > RShape::roundShapes(
         trimmed2 = QSharedPointer<RShape>(shape2->clone());
     }
 
-    RS::Ending ending1, ending2;
+    RS::Ending ending1 = RS::EndingNone;
+    RS::Ending ending2 = RS::EndingNone;
 
     if (trim || samePolyline) {
         // trim entities to intersection
@@ -2495,6 +2594,26 @@ QList<QSharedPointer<RShape> > RShape::roundShapes(
         return QList<QSharedPointer<RShape> >() <<  QSharedPointer<RShape>(new RPolyline(pl));
     }
 
+    if (RMath::fuzzyAngleCompare(arc.getStartAngle(), arc.getEndAngle())) {
+        // rounding is full circle (two shapes are tangential):
+        return ret;
+    }
+
+    if (RShape::isArcShape(*trimmed1)) {
+        QSharedPointer<RArc> arc1 = trimmed1.dynamicCast<RArc>();
+        if (RMath::fuzzyAngleCompare(arc1->getStartAngle(), arc1->getEndAngle())) {
+            // trimmed shape is full circle (two shapes are tangential):
+            return ret;
+        }
+    }
+    if (RShape::isArcShape(*trimmed2)) {
+        QSharedPointer<RArc> arc2 = trimmed2.dynamicCast<RArc>();
+        if (RMath::fuzzyAngleCompare(arc2->getStartAngle(), arc2->getEndAngle())) {
+            // trimmed shape is full circle (two shapes are tangential):
+            return ret;
+        }
+    }
+
     ret.append(trimmed1);
     ret.append(QSharedPointer<RShape>(new RArc(arc)));
     ret.append(trimmed2);
