@@ -92,6 +92,10 @@ public:
     virtual ~RShape() {
     }
 
+    virtual bool isValid() const {
+        return true;
+    }
+
     virtual RShape::Type getShapeType() const {
         return Unknown;
     }
@@ -130,6 +134,10 @@ public:
 
     virtual RShape* clone() const = 0;
 
+    virtual QSharedPointer<RShape> cloneToSharedPointer() const {
+        return QSharedPointer<RShape>(clone());
+    }
+
     /**
      * Reimplement and return true if this shape relies on interpolation
      * for geometrical operations such as finding intersection points. E.g.
@@ -260,7 +268,8 @@ public:
     QList<RVector> getIntersectionPoints(const RShape& other,
             bool limited = true, bool same = false, bool force = false) const;
 
-    virtual QList<RVector> getSelfIntersectionPoints() const {
+    virtual QList<RVector> getSelfIntersectionPoints(double tolerance=RS::PointTolerance) const {
+        Q_UNUSED(tolerance)
         return QList<RVector>();
     }
 
@@ -379,6 +388,7 @@ public:
 
     virtual QSharedPointer<RShape> getTransformed(const QTransform& transform) const = 0;
 
+    static QList<QSharedPointer<RShape> > getOrderedShapes(const QList<QSharedPointer<RShape> >& shapes);
     static bool order(QList<QList<QSharedPointer<RShape> > >& boundary);
 
     static QList<RVector> getIntersectionPointsLL(const RLine& line1,
@@ -447,7 +457,7 @@ public:
             const RExplodable& explodable2, bool limited);
 
     static QList<RVector> getIntersectionPointsSS(const RSpline& spline1,
-            const RSpline& spline2, bool limited = true, bool same = false);
+            const RSpline& spline2, bool limited = true, bool same = false, double tolerance=RS::PointTolerance);
 
     static QList<RVector> getIntersectionPointsXX(const RExplodable& explodable1,
             const RExplodable& explodable2, bool limited = true, bool same = false);
@@ -468,7 +478,7 @@ public:
             const RShape& limitingShape, const RVector& limitingClickPos,
             bool trimBoth, bool samePolyline);
 
-    static QList<QSharedPointer<RShape> > roundAllCorners(const QList<QSharedPointer<RShape> >& shapes, double radius);
+    static QList<QSharedPointer<RShape> > roundCorners(const QList<QSharedPointer<RShape> >& shapes, double radius);
 
     static QList<QSharedPointer<RShape> > roundShapes(
             const QSharedPointer<RShape> shape1, const RVector& clickPos1,
