@@ -36,6 +36,7 @@
 #include "RShapesExporter.h"
 #include "RSpline.h"
 #include "RTextLabel.h"
+#include "RTriangle.h"
 #include "RUnit.h"
 #include "RViewportEntity.h"
 
@@ -139,6 +140,7 @@ bool RGraphicsSceneQt::beginPath() {
     if (!layer.isNull()) {
         if (layer->getCustomBoolProperty("QCAD", "ScreenBasedLinetypes", false)==true) {
             screenBasedLinetypesOverride = true;
+            currentPainterPath.setScreenBasedLinetype(screenBasedLinetypesOverride);
         }
         // TODO: handle layer opacity:
 //        if (!layer->isPlottable()) {
@@ -272,20 +274,29 @@ void RGraphicsSceneQt::transformAndApplyPatternPath(RPainterPath& path) const {
         QList<QSharedPointer<RShape> > pathShapes = path.getShapes();
 
         lp.scale(getLineTypePatternScale(lp));
+
+        // detect viewport context:
+        double scaleHint = 1.0;
+        RViewportEntity* vp = getCurrentViewport();
+        if (vp!=NULL) {
+            scaleHint = vp->getScale();
+        }
+
         QPainterPath pathWithPattern;
+        RPainterPathExporter ppe(getDocument());
+        ppe.setPixelSizeHint(getPixelSizeHint());
+        ppe.setExportZeroLinesAsPoints(false);
+        ppe.setLinetypePattern(lp);
+        ppe.setIgnoreLineTypePatternScale(true);
+        ppe.setScaleHint(scaleHint);
 
         if (path.getPolylineGen()) {
-            RPainterPathExporter ppe(getDocument());
-            ppe.setPixelSizeHint(getPixelSizeHint());
-            ppe.setExportZeroLinesAsPoints(false);
-            ppe.setLinetypePattern(lp);
-            ppe.setIgnoreLineTypePatternScale(true);
 
             double length = 0.0;
             for (int i=0; i<pathShapes.length(); i++) {
                 length += pathShapes[i]->getLength();
             }
-            //qDebug() << "pattern offet:" << lp.getPatternOffset(length);
+            //qDebug() << "pattern offset:" << lp.getPatternOffset(length);
             RShapesExporter(ppe, pathShapes, lp.getPatternOffset(length));
             //lp.getPatternOffset(length));
             RPainterPath p = ppe.getPainterPath();
@@ -293,11 +304,12 @@ void RGraphicsSceneQt::transformAndApplyPatternPath(RPainterPath& path) const {
             pathWithPattern.addPath(p);
         }
         else {
-            RPainterPathExporter ppe(getDocument());
-            ppe.setPixelSizeHint(getPixelSizeHint());
-            ppe.setExportZeroLinesAsPoints(false);
-            ppe.setLinetypePattern(lp);
-            ppe.setIgnoreLineTypePatternScale(true);
+            //RPainterPathExporter ppe(getDocument());
+            //ppe.setPixelSizeHint(getPixelSizeHint());
+            //ppe.setExportZeroLinesAsPoints(false);
+            //ppe.setLinetypePattern(lp);
+            //ppe.setIgnoreLineTypePatternScale(true);
+            //ppe.setScaleHint(scaleHint);
 
             for (int i=0; i<pathShapes.length(); i++) {
                 //double length = 0.0;
@@ -317,7 +329,7 @@ void RGraphicsSceneQt::transformAndApplyPatternPath(RPainterPath& path) const {
                     //length = pathShapes[i]->getLength();
                 }
 
-                //qDebug() << "pattern offet:" << lp.getPatternOffset(length);
+                //qDebug() << "pattern offset:" << lp.getPatternOffset(length);
                 RShapesExporter(ppe, shapes, offset);
                 //lp.getPatternOffset(length));
                 RPainterPath p = ppe.getPainterPath();
@@ -462,6 +474,7 @@ void RGraphicsSceneQt::exportSpline(const RSpline& spline, double offset) {
     bool created = beginPath();
 
     RGraphicsScene::exportSpline(spline, offset);
+    currentPainterPath.setPolylineGen(true);
 
     if (created) {
         endPath();
@@ -500,18 +513,20 @@ void RGraphicsSceneQt::exportArcSegment(const RArc& arc, bool allowForZeroLength
             currentPainterPath.lineTo(arc.getEndPoint());
         }
         else {
-            if (getLinetypePattern().getNumDashes()<=1) {
-                // this is faster for continuous arcs:
-                currentPainterPath.setAutoRegen(true);
-                RGraphicsScene::exportArcSegment(arc, allowForZeroLength);
-            }
-            else {
-                // slower:
-                currentPainterPath.setAutoRegen(true);
-                QList<RSpline> splines = RSpline::createSplinesFromArc(arc);
-                for (int i=0; i<splines.length(); i++) {
-                    currentPainterPath.addSpline(splines[i]);
-                }
+            // this is faster for continuous arcs:
+            // 20210903: also use this for arcs with patterns for screen-based linetypes:
+            currentPainterPath.setAutoRegen(true);
+            RGraphicsScene::exportArcSegment(arc, allowForZeroLength);
+
+//            if (getLinetypePattern().getNumDashes()<=1) {
+//            }
+//            else {
+//                // slower:
+//                currentPainterPath.setAutoRegen(true);
+//                QList<RSpline> splines = RSpline::createSplinesFromArc(arc);
+//                for (int i=0; i<splines.length(); i++) {
+//                    currentPainterPath.addSpline(splines[i]);
+//                }
 
                 // this is not precise enough:
 //                currentPainterPath.arcTo(
@@ -521,7 +536,7 @@ void RGraphicsSceneQt::exportArcSegment(const RArc& arc, bool allowForZeroLength
 //                    RMath::rad2deg(-arc.getStartAngle()),
 //                    RMath::rad2deg(-arc.getSweep())
 //                );
-            }
+//            }
         }
     }
     else {
@@ -558,6 +573,9 @@ void RGraphicsSceneQt::exportLineSegment(const RLine& line, double angle) {
     }
 
     currentPainterPath.lineTo(line.endPoint);
+
+    //currentPainterPath.setPixelWidth(true);
+    //currentPainterPath.setPen(QPen(QBrush("white"), 10));
 }
 
 void RGraphicsSceneQt::exportXLine(const RXLine& xLine) {
@@ -904,10 +922,16 @@ void RGraphicsSceneQt::deleteDrawables() {
     previewClipRectangles.clear();
 }
 
+QList<RGraphicsSceneDrawable> RGraphicsSceneQt::getDrawablesList(REntity::Id entityId) {
+    if (drawables.contains(entityId)) {
+        return drawables[entityId];
+    }
+    return QList<RGraphicsSceneDrawable>();
+}
+
 /**
- * \return A list of all painter paths that represent the entity with the
+ * \return A list of all drawables (painter path, text, image) that represent the entity with the
  * given ID.
- * TODO: return reference or pointer
  */
 QList<RGraphicsSceneDrawable>* RGraphicsSceneQt::getDrawables(REntity::Id entityId) {
     // TODO: check should not be necessary:
@@ -1034,9 +1058,18 @@ bool RGraphicsSceneQt::hasPreview() const {
 }
 
 QList<REntity::Id> RGraphicsSceneQt::getPreviewEntityIds() {
-    QList<REntity::Id> ret = previewDrawables.keys();
-    ret.append(previewClipRectangles.keys());
-    ret = ret.toSet().toList();
+    QList<REntity::Id> retWithDuplicates = previewDrawables.keys();
+    retWithDuplicates.append(previewClipRectangles.keys());
+
+    // remove duplicate without changing order:
+    QSet<REntity::Id> set;
+    QList<REntity::Id> ret;
+    for (int i=0; i<retWithDuplicates.length(); i++) {
+        if (!set.contains(retWithDuplicates[i])) {
+            ret.append(retWithDuplicates[i]);
+        }
+    }
+
     return ret;
 }
 
@@ -1052,9 +1085,9 @@ void RGraphicsSceneQt::clearPreview() {
     previewDrawables.clear();
 }
 
-void RGraphicsSceneQt::exportEntities(bool allBlocks, bool undone) {
+void RGraphicsSceneQt::exportEntities(bool allBlocks, bool undone, bool invisible) {
     //RDebug::startTimer(100);
-    RGraphicsScene::exportEntities(allBlocks, undone);
+    RGraphicsScene::exportEntities(allBlocks, undone, invisible);
     //RDebug::stopTimer(100, "exportEntities");
 
     /*
@@ -1218,7 +1251,7 @@ void RGraphicsSceneQt::startEntity(bool topLevelEntity) {
  * Stream operator for QDebug
  */
 QDebug operator<<(QDebug dbg, RGraphicsSceneQt& gs) {
-    dbg.nospace() << "RGraphicsSceneQt(" << QString("%1").arg((long int)&gs, 0, 16) << ")";
+    dbg.nospace() << "RGraphicsSceneQt(" << QString("%1").arg((unsigned long long int)&gs, 0, 16) << ")";
     //QMap<REntity::Id, QList<RGraphicsSceneDrawable> >::iterator it;
     //for (it=gs.drawables.begin(); it!=gs.drawables.end(); it++) {
         //dbg.nospace() << "\n" << it.key() << "\n  " << it.value() << "\n";
