@@ -347,8 +347,7 @@ void RGraphicsViewImage::updateImage() {
 
     // paint reference points of selected entities:
     QMap<REntity::Id, QList<RRefPoint> >& referencePoints = scene->getReferencePoints();
-    int num = scene->countReferencePoints();
-    if (num!=0 && num<RSettings::getIntValue("GraphicsView/MaxReferencePoints", 100000)) {
+    if (getDocument()->countSelectedEntities()<RSettings::getMaxReferencePointEntitiesDisplay()) {
         QPainter gbPainter(&graphicsBufferWithPreview);
         QMap<REntity::Id, QList<RRefPoint> >::iterator it;
         for (it = referencePoints.begin(); it != referencePoints.end(); ++it) {
@@ -387,6 +386,7 @@ void RGraphicsViewImage::updateImage() {
             RSnap* snap = di->getSnap();
             RSnapRestriction* snapRestriction = di->getSnapRestriction();
             emitUpdateSnapInfo(snap, snapRestriction);
+            // prevent snap to show "End" directly after snapping to grid:
             if (snap!=NULL) {
                 snap->reset();
             }
@@ -862,6 +862,7 @@ void RGraphicsViewImage::paintBackground(QPainter* painter, const QRect& rect) {
             double boxTop;
 
             switch (text.getVAlign()) {
+            default:
             case RS::VAlignTop:
                 flags|=Qt::AlignTop;
                 boxTop = -pos.y;
@@ -873,6 +874,7 @@ void RGraphicsViewImage::paintBackground(QPainter* painter, const QRect& rect) {
             }
 
             switch (text.getHAlign()) {
+            default:
             case RS::HAlignLeft:
                 flags|=Qt::AlignLeft;
                 boxLeft = pos.x;
@@ -977,7 +979,15 @@ void RGraphicsViewImage::paintOverlay(QPainter* painter) {
 
                     if (drawable.getPixelUnit()) {
                         //text.scale(RVector(1/factor,1/factor), text.getAlignmentPoint());
-                        text.scale(RVector(1/factor,1/factor), RVector(0,0));
+
+                        if (!isPrinting() && !isPrintPreview()) {
+                            text.scale(RVector(1/factor,1/factor), RVector(0,0));
+                        }
+                        else {
+                            double f = RUnit::convert(0.22, RS::Millimeter, getDocument()->getUnit());
+                            f/=getDevicePixelRatio();
+                            text.scale(RVector(f,f), RVector(0,0));
+                        }
                     }
 
                     text.move(drawable.getOffset());
@@ -1131,6 +1141,7 @@ void RGraphicsViewImage::paintEntitiesMulti(const RBox& queryBox) {
 
         // regenerate arc, xline, block reference, etc.:
         if (regen) {
+            // TODO: only regen part of block ref / viewport that needs regeneration:
             sceneQt->exportEntity(id, true);
         }
     }
@@ -1145,21 +1156,27 @@ void RGraphicsViewImage::paintEntitiesMulti(const RBox& queryBox) {
         int start = threadId*slice;
         int end = (threadId+1)*slice;
         if (threadId==painterThread.length()-1) {
-            end = list.length();
+            end = (int)list.length();
         }
         //qDebug() << "slice:" << start << end;
-        futureThread.append(QtConcurrent::run(this, &RGraphicsViewImage::paintEntitiesThread, threadId, list, start, end));
+
+        // TODO (#qt6):
+        //futureThread.append(QtConcurrent::run(this, &RGraphicsViewImage::paintEntitiesThread, threadId, list, start, end));
+        //QFuture<void> future = QtConcurrent::run(&RGraphicsViewImage::paintEntitiesThread, *this, threadId, list, start, end);
+        //futureThread.append(future);
+
+        paintEntitiesThread(threadId, list, start, end);
     }
     //RDebug::stopTimer(100, "launch threads");
 
     //RDebug::startTimer(100);
-    for (int i=0; i<futureThread.length(); i++) {
-        futureThread[i].waitForFinished();
-    }
+//    for (int i=0; i<futureThread.length(); i++) {
+//        futureThread[i].waitForFinished();
+//    }
     //RDebug::stopTimer(100, "waitForFinished");
 }
 
-void RGraphicsViewImage::paintEntitiesThread(int threadId, QList<REntity::Id>& list, int start, int end) {
+void RGraphicsViewImage::paintEntitiesThread(int threadId, const QList<REntity::Id>& list, int start, int end) {
     for (int i=start; i<end; i++) {
         paintEntityThread(threadId, list[i]);
     }
@@ -1276,7 +1293,10 @@ void RGraphicsViewImage::paintEntityThread(int threadId, REntity::Id id, bool pr
 
         // drawable is not plottable (from layer for which plottable is off):
         if (drawable.getNoPlot() && (isPrinting() || (showOnlyPlottable && isPrintPreview()))) {
-            continue;
+            // always export transforms (viewports):
+            if (drawable.getType()!=RGraphicsSceneDrawable::Transform && drawable.getType()!=RGraphicsSceneDrawable::EndTransform) {
+                continue;
+            }
         }
 
         bool workingSet = true;
@@ -1584,7 +1604,8 @@ void RGraphicsViewImage::paintEntityThread(int threadId, REntity::Id id, bool pr
         painter->setBrush(brush);
         painter->setPen(pen);
 
-        if (scene->getDraftMode() || isPrintingOrExporting() /*|| clipBox.contains(pathBB)*/) {
+        // 20210903: make sure screen-based linetypes are rendered as path to keep pattern visible:
+        if (scene->getScreenBasedLinetypes() || path.getScreenBasedLinetype() || scene->getDraftMode() || isPrintingOrExporting() /*|| clipBox.contains(pathBB)*/) {
             if (brush.style() != Qt::NoBrush) {
                 painter->fillPath(path, brush);
             }
@@ -1688,6 +1709,7 @@ void RGraphicsViewImage::paintEntityThread(int threadId, REntity::Id id, bool pr
 
         // draw points:
         if (path.hasPoints()) {
+            // simple point display for hatches (don't apply point styles):
             if (path.getSimplePointDisplay()) {
                 double ps = getScene()->getPixelSizeHint();
                 // simple point mode for hatch patterns:
@@ -1718,7 +1740,9 @@ void RGraphicsViewImage::paintEntityThread(int threadId, REntity::Id id, bool pr
                 // the lines don't turn into a blob
                 // This also applies when exporting (e.g. to bitmap):
                 QPen pen = painter->pen();
-                pen.setWidth(0);
+                if (!RSettings::getApplyLineweightToPoints()) {
+                    pen.setWidth(0);
+                }
                 painter->setPen(pen);
             }
 
@@ -1992,11 +2016,20 @@ void RGraphicsViewImage::paintImage(QPainter* painter, RImageData& image, bool w
         double angle = image.getUVector().getAngle();
 
         painter->save();
+        /*
         QMatrix wm = painter->matrix();
         wm.translate(image.getInsertionPoint().x, image.getInsertionPoint().y);
         wm.rotate(RMath::rad2deg(angle));
         wm.scale(scale.x, -scale.y);
         painter->setMatrix(wm);
+        */
+
+        QTransform t = painter->worldTransform();
+        t.translate(image.getInsertionPoint().x, image.getInsertionPoint().y);
+        t.rotate(RMath::rad2deg(angle));
+        t.scale(scale.x, -scale.y);
+        painter->setWorldTransform(t);
+
         double opacity = 1.0;
         if (image.getFade()>0 && image.getFade()<100) {
             opacity = 1.0 - ((double)image.getFade()/100);
