@@ -36,6 +36,7 @@
 #include "RDimOrdinateEntity.h"
 #include "RDimRadialEntity.h"
 #include "RDimRotatedEntity.h"
+#include "RDimStyle.h"
 #include "RDimensionEntity.h"
 #include "RDxfExporter.h"
 #include "REllipseEntity.h"
@@ -117,6 +118,9 @@ bool RDxfExporter::exportFile(const QString& fileName, const QString& nameFilter
 
         // Variables
         //qDebug() << "RDxfExporter::exportFile: variables";
+        QSharedPointer<RDimStyle> dimStyle = document->queryDimStyleDirect();
+        dimStyle->updateDocumentVariables();
+
         writeVariables();
 
         // end header
@@ -143,7 +147,7 @@ bool RDxfExporter::exportFile(const QString& fileName, const QString& nameFilter
         }
     }
     else {
-        QStringList lts = document->getLinetypeNames().toList();
+        QStringList lts = RS::toList(document->getLinetypeNames());
         //qDebug() << "RDxfExporter::exportFile: linetypes table";
         dw->tableLinetypes(lts.size());
         // continuous must always be the first LTYPE:
@@ -172,7 +176,7 @@ bool RDxfExporter::exportFile(const QString& fileName, const QString& nameFilter
         layerNames.append("0");
     }
     else {
-        layerNames = document->getLayerNames().toList();
+        layerNames = RS::toList(document->getLayerNames());
     }
     dw->tableLayers(layerNames.size());
     for (int i=0; i<layerNames.size(); ++i) {
@@ -211,7 +215,7 @@ bool RDxfExporter::exportFile(const QString& fileName, const QString& nameFilter
         uniqueTextStyles.append(style);
 
         // write one text style for each new, unique combination of font, size, etc:
-        QList<REntity::Id> entityIds = document->queryAllEntities(false, true).toList();
+        QList<REntity::Id> entityIds = RS::toList(document->queryAllEntities(false, true));
         for (int i=0; i<entityIds.size(); i++) {
             REntity::Id id = entityIds[i];
             QSharedPointer<REntity> entity = document->queryEntityDirect(id);
@@ -266,17 +270,32 @@ bool RDxfExporter::exportFile(const QString& fileName, const QString& nameFilter
     // DIMSTYLE:
     if (!minimalistic) {
         //qDebug() << "writing dim styles...";
-        dxf.writeDimStyle(*dw,
-                          document->getKnownVariable(RS::DIMASZ, 2.5).toDouble(),
-                          document->getKnownVariable(RS::DIMEXE, 0.625).toDouble(),
-                          document->getKnownVariable(RS::DIMEXO, 0.625).toDouble(),
-                          document->getKnownVariable(RS::DIMGAP, 0.625).toDouble(),
-                          document->getKnownVariable(RS::DIMTXT, 2.5).toDouble()
-        );
+        QSharedPointer<RDimStyle> dimStyle = document->queryDimStyleDirect();
+        if (!dimStyle.isNull()) {
+            dxf.writeDimStyle(*dw,
+                              dimStyle->getDouble(RS::DIMASZ),
+                              dimStyle->getDouble(RS::DIMEXE),
+                              dimStyle->getDouble(RS::DIMEXO),
+                              dimStyle->getDouble(RS::DIMGAP),
+                              dimStyle->getDouble(RS::DIMTXT),
+                              dimStyle->getInt(RS::DIMTAD),
+                              dimStyle->getBool(RS::DIMTIH)
+            );
+        }
+
+//        dxf.writeDimStyle(*dw,
+//                          document->getKnownVariable(RS::DIMASZ, 2.5).toDouble(),
+//                          document->getKnownVariable(RS::DIMEXE, 0.625).toDouble(),
+//                          document->getKnownVariable(RS::DIMEXO, 0.625).toDouble(),
+//                          document->getKnownVariable(RS::DIMGAP, 0.625).toDouble(),
+//                          document->getKnownVariable(RS::DIMTXT, 2.5).toDouble(),
+//                          document->getKnownVariable(RS::DIMTAD, 1).toInt(),
+//                          document->getKnownVariable(RS::DIMTIH, false).toBool()
+//        );
     }
 
     // BLOCK_RECORD:
-    QStringList blockNames = document->getBlockNames().toList();
+    QStringList blockNames = RS::toList(document->getBlockNames());
     if (exportVersion!=DL_Codes::AC1009 && exportVersion!=DL_Codes::AC1009_MIN) {
         //qDebug() << "writing block records...";
         dxf.writeBlockRecord(*dw);
@@ -368,33 +387,33 @@ bool RDxfExporter::exportFile(const QString& fileName, const QString& nameFilter
                 QString key = variables[i];
                 QVariant value = document->getVariable(key);
                 if (handles.contains(key)) {
-                    switch (value.type()) {
-                    case QVariant::Int:
+                    switch (RS::getMetaType(value)) {
+                    case RS::Int:
                         dxf.writeXRecord(*dw, handles.value(key), value.toInt());
                         break;
-                    case QVariant::Double:
+                    case RS::Double:
                         dxf.writeXRecord(*dw, handles.value(key), value.toDouble());
                         break;
-                    case QVariant::Bool:
+                    case RS::Bool:
                         dxf.writeXRecord(*dw, handles.value(key), value.toBool());
                         break;
-                    case QVariant::String:
+                    case RS::String:
                         dxf.writeXRecord(*dw, handles.value(key), std::string((const char*)RDxfExporter::escapeUnicode(value.toString())));
                         break;
-                    case QVariant::Font:
+                    case RS::Font:
                         if (value.canConvert<QFont>()) {
                             QFont f = value.value<QFont>();
                             dxf.writeXRecord(*dw, handles.value(key), std::string((const char*)RDxfExporter::escapeUnicode(f.toString())));
                         }
                         break;
-                    case QVariant::UserType:
+                    case RS::UserType:
                         if (value.canConvert<RColor>()) {
                             RColor c = value.value<RColor>();
                             dxf.writeXRecord(*dw, handles.value(key), std::string((const char*)RDxfExporter::escapeUnicode(c.getName())));
                         }
                         break;
                     default:
-                        qWarning() << "RDxfExporter::exportFile: unsupported extension data type: " << value.type();
+                        qWarning() << "RDxfExporter::exportFile: unsupported extension data type: " << RS::getMetaType(value);
                         qWarning() << value;
                         //Q_ASSERT(false);
                         break;
@@ -480,20 +499,21 @@ void RDxfExporter::writeVariables() {
 
         name = "$" + name;
 
-        switch (value.type()) {
-        case QVariant::Int:
+        switch (RS::getMetaType(value)) {
+        case RS::Int:
+        case RS::Bool:
             dw->dxfString(9, (const char*)RDxfExporter::escapeUnicode(name));
             dw->dxfInt(code, value.toInt());
             break;
-        case QVariant::Double:
+        case RS::Double:
             dw->dxfString(9, (const char*)RDxfExporter::escapeUnicode(name));
             dw->dxfReal(code, value.toDouble());
             break;
-        case QVariant::String:
+        case RS::String:
             dw->dxfString(9, (const char*)RDxfExporter::escapeUnicode(name));
             dw->dxfString(code, (const char*)RDxfExporter::escapeUnicode(value.toString()));
             break;
-        case QVariant::UserType:
+        case RS::UserType:
             if (value.canConvert<RVector>()) {
                 RVector v = value.value<RVector>();
                 dw->dxfString(9, (const char*)RDxfExporter::escapeUnicode(name));
@@ -1219,7 +1239,7 @@ void RDxfExporter::writeDimension(const RDimensionEntity& d) {
                              (const char*)RDxfExporter::escapeUnicode(d.getFontName()),
                              d.getTextAngle(),
                              d.getLinearFactor(),
-                             d.getDimScale());
+                             d.getDimscale());
     dimData.arrow1Flipped = d.isArrow1Flipped();
     dimData.arrow2Flipped = d.isArrow2Flipped();
 
@@ -1344,7 +1364,7 @@ void RDxfExporter::writeLeader(const RLeaderEntity& l) {
                       1.0,
                       10.0,
                       l.countVertices(),
-                      l.getDimScale());
+                      l.getDimscale());
 
         dxf.writeLeader(
             *dw,
